/*
 * twcw:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * Versions: 1.1 -  MAR 2003
 */

#include "common.h"
#include "sendCW.h"

int arg = TIOCM_RTS;
int mkbeep = 0;



/*
 * SetTone FUNCTION
 * sets sidetone freq
 */
void SetTone (int new_tone)
{
   tone = 1193180 / new_tone;
}


/*
 * SetSidetone FUNCTION
 * turns sidetone on or off
 */
void SetSidetone (int new_tone)
{
   mkbeep = new_tone;
}


/*
 * DoWord FUNCTION
 */
void DoWord (unsigned char w[], float cpu_clock, int key, int rdtsc_ok)
{
   int  i = 0;
   struct tag1 reply;

   while (w[i] != (char) NULL)
   {
      if (letters[(int) w[i]] == (char) NULL)  /* Unknown letter, so skip it */
      {
         i++;
         continue;
      }

      /* Send this letter */
      DoLetter (letters[(unsigned int) w[i]], cpu_clock, key, rdtsc_ok);

      /*If key == ON, then send this character back to the GUI */
      if (key == ON)
      {
         reply.type = SENT_MSG;
         reply.data.word[0] = w[i];
         reply.data.word[1] = (char) NULL;
         if (msgsnd (qid, (struct msgbuf *)&reply, sizeof(reply.data), 0) == -1)
         {
            perror ("msgsnd SENT_MSG");
            exit (1);
         }
      }
      i++;
   }
   timer (iw_time, cpu_clock, rdtsc_ok);
   /* Done with a word, so send a space to GUI */
   reply.type = SENT_MSG;
   reply.data.word[0] = ' ';
   reply.data.word[1] = (char) NULL;

   if (msgsnd (qid, (struct msgbuf *) &reply, sizeof (reply.data), 0) == -1)
   {
      perror ("msgsnd SENT_MSG");
      exit (1);
   }
}


/*
 * DoLetter FUNCTION
 */
void DoLetter (char ch[], float cpu_clock, int key, int rdtsc_ok)
{
   int i = 0;

   while (ch[i] != (char) NULL)
   {
      if (ch[i] == DIT)
         PlayBeep (dit_time, cpu_clock, key, rdtsc_ok);
      if (ch[i] == DAH)
         PlayBeep (dah_time, cpu_clock, key, rdtsc_ok);
      i++;
   }
   timer (il_time, cpu_clock, rdtsc_ok);
}


/*
 * PlayBeep FUNCTION
 */
void PlayBeep (int time, float cpu_clock, int key, int rdtsc_ok)
{
   if (mkbeep == ON)   /* Start tone if mkbeep is ON */
   {
      if (ioctl (confd, KIOCSOUND, tone) == -1)
         perror("KIOCSOUND1");
   }

   if (key == ON)     /* Toggle RTS if key is ON */
   {
      if (ioctl (serfd, TIOCMBIS, &arg) == -1)
         perror("RTS failed");
   }
   else
   {
      if (ioctl (serfd, TIOCMBIC, &arg) == -1)
         perror("RTS failed"); 
   }
   timer (time, cpu_clock, rdtsc_ok);

   if (mkbeep == 1)
   {
      if (ioctl (confd, KIOCSOUND, 0) == -1)
         perror("KIOCSOUND2");
   }

   if (ioctl (serfd, TIOCMBIC, &arg) == -1)
         perror("RTS failed");

   timer (ic_time, cpu_clock, rdtsc_ok);
}


/*
 * SetWPM FUNCTION
 */
void SetWPM (int new_speed, float cpu_speed, int rdtsc_ok)
{
   int fudge;
   struct timeval cal_start, cal_stop;
   char *test_string = "PARIS";
   struct tag1 reply;
   int savebeep = mkbeep;

   wpm = new_speed;
   dit_time = 1200000 / wpm;            /* true dit time in usecs */
   ic_time = dit_time;
   dah_time = 3 * dit_time;
   il_time = dah_time;
   iw_time = 6 * dit_time;

   if (rdtsc_ok == 0)
   {   
      /* Using gettimeofday method so calibrate WPM by getting the start time,
       * "Sending" Paris, and getting the end time.  Calculate a fudge factor
       * by subtracting the calculated time from the measured time.
       */
      mkbeep = 0;
      if (gettimeofday (&cal_start, NULL) == 1)
      {
         perror("gettimeofday start");
         exit (1);
      }

      DoWord (test_string, cpu_speed, OFF, rdtsc_ok);

      if (gettimeofday (&cal_stop, NULL) == 1)
      {
         perror("gettimeofday stop");
         exit (1);
      }
      fudge = (((cal_stop.tv_sec - cal_start.tv_sec) * 1000000 
               + cal_stop.tv_usec - cal_start.tv_usec) / 50) - dit_time;
      printf ("fudge dit_time by = %d\n\n", fudge);
      mkbeep = savebeep;
      dit_time = dit_time - fudge; /* dit time using gettimeofday */
      ic_time = dit_time;
      dah_time = 3 * dit_time;
      il_time = dah_time;
      iw_time = 6 * dit_time;      /* Total of 7, since each letter is
                                    * followed by an ic_time
                                    */

      printf ("dit time = %ld\n", dit_time);
  }
   reply.type = WPM_DONE;       /* Tell the GUI we are done setting WPM */
   reply.data.word[0] = ' ';
   reply.data.word[1] = (char) NULL;
   if (msgsnd (qid, (struct msgbuf *) &reply, sizeof (reply.data), 0) == -1)
   {
      perror ("msgsnd WPM_DONE");
      exit (1);
   }
}


/*
 * initialize FUNCTION
 */
int initialize (float cpu_clock, char *ser_dev)/* add ser_dev string */
{
   int i;
   int rdtsc_ok = 0;

   /* 
    * Probe for clock
    */

   rdtsc_ok = probe_clock (cpu_clock);

   /*
    * Open the console for sidetone
    */

   if ((confd = open ("/dev/console", O_RDWR)) == -1)
   {
      perror ("Can't open console");
      exit (1);
   }

   /*
    * Open the serial port for keying and make sure the key is up
    */
   if ((serfd = open (ser_dev, O_RDWR)) == -1)
   {
      perror ("Can't open serial port");
      exit (1);
   }

   if (ioctl (serfd, TIOCMBIC, &arg) == -1)
   {
      perror("RTS failed");
      exit (1);
   }

   /*
    * Initialize the letters array
    */
   for (i = 0; i < MAX_LETTERS; i++)
      letters[i] = (char) NULL;

   /* Numbers */
   letters[(int) '1'] = ".----";
   letters[(int) '2'] = "..---";
   letters[(int) '3'] = "...--";
   letters[(int) '4'] = "....-";
   letters[(int) '5'] = ".....";
   letters[(int) '6'] = "-....";
   letters[(int) '7'] = "--...";
   letters[(int) '8'] = "---..";
   letters[(int) '9'] = "----.";
   letters[(int) '0'] = "-----";
   letters[216]       = "-----";        /* 216 or 330h is a zero with a slash */

   /* Alphabet */
   letters[(int) ' '] = "";
   letters[(int) 'A'] = ".-";
   letters[(int) 'B'] = "-...";
   letters[(int) 'C'] = "-.-.";
   letters[(int) 'D'] = "-..";
   letters[(int) 'E'] = ".";
   letters[(int) 'F'] = "..-.";
   letters[(int) 'G'] = "--.";
   letters[(int) 'H'] = "....";
   letters[(int) 'I'] = "..";
   letters[(int) 'J'] = ".---";
   letters[(int) 'K'] = "-.-";
   letters[(int) 'L'] = ".-..";
   letters[(int) 'M'] = "--";
   letters[(int) 'N'] = "-.";
   letters[(int) 'O'] = "---";
   letters[(int) 'P'] = ".--.";
   letters[(int) 'Q'] = "--.-";
   letters[(int) 'R'] = ".-.";
   letters[(int) 'S'] = "...";
   letters[(int) 'T'] = "-";
   letters[(int) 'U'] = "..-";
   letters[(int) 'V'] = "...-";
   letters[(int) 'W'] = ".--";
   letters[(int) 'X'] = "-..-";
   letters[(int) 'Y'] = "-.--";
   letters[(int) 'Z'] = "--..";

   /* Punctuation */
   letters[(int) '.'] = ".-.-.-";
   letters[(int) ','] = "--..--";
   letters[(int) '?'] = "..--..";
   letters[(int) '/'] = "-..-.";

   /* Pro Signs and Special */   
   letters[(int) '\n'] = "";
   letters[(int) ' '] = "";
   letters[(int) '-'] = "-...-";    /* - = <BT> */
   letters[(int) '!'] = "........"; /* ! = ERROR */
   letters[(int) '@'] = ".-.-.";    /* @ = <AR>  */
   letters[(int) '#'] = "...-.-";   /* # = <SK>  */
   letters[(int) '$'] = "-.--.";    /* $ = <KN>  */
   letters[(int) '%'] = ".-...";    /* % = WAIT  */
   letters[(int) '^'] = "-...-.-";  /* ^ = BREAK */

   return (rdtsc_ok);
}




/*
 * cleanup FUNCTION 
 * Make sure the sidetone is off and the key is up, then exit
 */
void cleanup (void)
{
   signal (SIGINT,  SIG_IGN);
   signal (SIGTERM, SIG_IGN);
   signal (SIGQUIT, SIG_IGN);
   signal (SIGSEGV, SIG_IGN);
   fprintf(stderr, "sendCW terminating\n");

   /* make sure sound is off and the key is up */
   if (confd != 0)
      ioctl (confd, KIOCSOUND, 0);

   if (serfd != 0)
      if (ioctl (serfd, TIOCMBIC, &arg) == -1)
         perror("RTS failed"); 

   exit (73);
}

