/*
 * twcw:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Versions: 1.1 -  MAR 2003
 */


/*
 * main FUNCTION 
 * child process that sends CW via RTS line on the serial port.
 */
#include "common.h"
#include "sendCW.h"

int  main (int argc, char *argv[])
{

   double cpu_clock = atof(argv[1]);
   char *ser_dev = argv[2];
   int  rdtsc_ok, rtn, mode = 0;
   struct tag1 message;

   /*
    * trap the following signals for the cleanup function
    */
   signal (SIGINT,  cleanup);
   signal (SIGTERM, cleanup);
   signal (SIGQUIT, cleanup);
   signal (SIGSEGV, cleanup);

   /*
    * Initialize letters array and connect to the message queue
    */
   rdtsc_ok = initialize (cpu_clock, ser_dev);

   if ((qid = msgget (TWCW_KEY, 0666)) < 0)
   {
      perror ("msgget");
      exit (1);
   }

   /*
    * Read a Message From the Queue
    */
   while (1)
   {
      rtn = msgrcv (qid, (struct msgbuf *) &message, MAX_WORD_LENGTH, mode, 0);
      switch (message.type)
      {
         case WORD_MSG:                      /* Got a word to send */
            DoWord (message.data.word, cpu_clock, ON, rdtsc_ok);
            continue;

         case WPM_MSG:                       /* Sets a new WPM */
            SetWPM (message.data.val, cpu_clock, rdtsc_ok);
            continue;

         case TONE_MSG:
            SetTone (message.data.val);      /* Sets the tone freq */
            continue;

         case SIDETONE_MSG:
            SetSidetone (message.data.val);  /* Turns sidetone on and off */
            continue;

         case STOP_MSG:
            if (message.data.word[0] == '0')
               mode = -STOP_MSG;             /* Stop sending */ 
            else
               mode = -WORD_MSG;             /* Start sending */
      }
   }
   exit (0);
}
