/*
 * twcwCB.c:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 -  MAR 2003
 */

#include "common.h"
#include "twcw.h"
#include <sys/shm.h>

#define ON "1"
#define OFF "0"
#define XMIT "1"
#define STANDBY "0"
#define TEXT  0
#define INSERT 1

extern AppRes app_res;

/*
 * text1CB Function
 * Modify/Verify Callback for text1 (Main Text Widget)
 */
static int j;                           /* position in current word */
static char savech;
long int old_end = -1;

void text1CB (Widget w, XtPointer client_data, XtPointer cdata)
{
   int  i;
   static char word[MAX_WORD_LENGTH];
   XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *) cdata;

   if (cbs->reason == XmCR_VALUE_CHANGED)
   {
      XmTextSetInsertionPosition (w, XmTextGetLastPosition (w));
      return;
   }

   if (cbs->startPos <= old_end)	/* Don't allow Big Delete */
   {
      cbs->doit = False;  
      XmTextSetInsertionPosition (w, XmTextGetLastPosition (w));
      return;
   }

   if (cbs->endPos - cbs->startPos == 1) /* Allow Back Space */
   {
      j--;
      return;
   }

   for (i = 0; i < cbs->text->length; i++)
   {                                    /* Now all things to upper case */
      cbs->text->ptr[i] = toupper (cbs->text->ptr[i]);
      word[j] = cbs->text->ptr[i];

      if (word[j] == ' ' || word[j] == '\n')
      {
         savech = word[j];
         word[j] = '\0';
         send_msg (WORD_MSG, word);     /* Send the word to the queue */
         word[j] = savech;
         j = 0;
         old_end = cbs->endPos;
      }
      else
      {
         if (cbs->text->ptr[i] == '0')
         {
            cbs->text->ptr[i] = '\330'; /* make zeros have a slash */
            word[j] = '\330';
         }
         i = DoProSigns(cbs, i);        /* check for pro signs */
         j++;
      }
    }
}

/*
 * textf1CB Function - If its a value changed CB, set the cursor to
 * the last position.  Otherwise, see if we gota Pro Sign and add it
 * to the end of the text.
 */
void textf1CB (Widget w,XtPointer client_data, XtPointer call_data)
{
   int i;
   XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *) call_data;

   if (cbs->reason == XmCR_VALUE_CHANGED)
   {
      XmTextSetInsertionPosition (w, XmTextGetLastPosition (w));
      return;
   }

   for (i = 0; i < cbs->text->length; i++)
   {
      i = DoProSigns(cbs, i);           /* check for pro signs */
   }
}


/*
 * DoProSign Function
 * Check for a pro signs character in the text string and
 * if found, insert pro sign char strings in the text widgets.
 * Used by text1 and textf1 modify/verify callback.
 */
int DoProSigns(XmTextVerifyCallbackStruct *cbs, int i)
{
  switch (cbs->text->ptr[i])
  {
      case '@':
         MkRoom (cbs, i);
         cbs->text->ptr[i++] = 'a';     /* @ will be printed as "ar" */
         cbs->text->ptr[i] = 'r';
         break;

      case '#':
         MkRoom (cbs, i);               /* # will be printed as "sk" */
         cbs->text->ptr[i++] = 's';
         cbs->text->ptr[i] = 'k';
         break;

      case '$':
         MkRoom (cbs, i);               /* $ will be printed as "kn" */
         cbs->text->ptr[i++] = 'k';
         cbs->text->ptr[i] = 'n';
         break;

      case '%':
         MkRoom (cbs, i);               /* % will be printed as "as" */
         cbs->text->ptr[i++] = 'a';
         cbs->text->ptr[i] = 's';
         break;

      case '^':
         MkRoom (cbs, i);               /* ^ will be printed as "bk" */
         cbs->text->ptr[i++] = 'b';
         cbs->text->ptr[i] = 'k';
         break;

      case '-':
         MkRoom (cbs, i);               /* - will be printed as "bt" */
         cbs->text->ptr[i++] = 'b';
         cbs->text->ptr[i] = 't';
         break;

      default:
         break;
   }
   return i;
}


/*
 * MkRoom FUNCTION
 * Called by DoProSigns if a pro sign character is found in a text string.
 * This will make room for the pro sign's two character string in the text
 * widgit.
 */
void MkRoom (XmTextVerifyCallbackStruct *cbs, int i)
{
   int k;

   cbs->text->ptr = XtRealloc (cbs->text->ptr, cbs->text->length + 2);
   cbs->text->length = cbs->text->length + 1;
   for (k=cbs->text->length; k>i+1; k--)
   {
      cbs->text->ptr[k] = cbs->text->ptr[k-1];
   }
   return;
}


/*
 * fileCB FUNCTION
 * Activate Callback for File Pulldown
 */
void fileCB (Widget w, XtPointer client_data, XtPointer call_data)
{
   int btn = (int) client_data;
   AppRes *resptr;
   char *filename;
   char *text;

   /*
    *    Get the dir path and filename, malloc space, and cat them   
    */
   XtVaGetValues (w,
      XmNuserData, (AppRes *) &resptr,
      NULL);

   filename = (char *) XtMalloc (strlen (dirpath)
                               + strlen (resptr->buttonNames[btn]) + 2);
   strcpy (filename, dirpath);
   strcat (filename, "/");            /* could have // but doesn't hurt */
   strcat (filename, resptr->buttonNames[btn]);

   /* 
    * Get file contents and call the work proc to process it.
    */
   text = getFile (filename);                          
   XtAppAddWorkProc (ac, procData, (XtPointer) text);  

   XtFree (filename);
} 


/*
 * getFile Function
 * Passed a pathname, and a char *, this malloc's space for the file
 * text and reads the file.
 */
char *getFile (char *filename)
{
   FILE *fp;
   struct stat info;
   char *pt;
   if (stat (filename, &info) != 0)
   {
       perror ("File Stat");
       exit (1);
   }

   if ((pt = (char *)XtMalloc ((int)info.st_size + 2)) == NULL)
   {
       perror ("Xtmalloc");
       exit (1);
   }

   if ((fp = fopen (filename, "r")) == NULL)   
   {
       perror ("File Open");
       exit (1);
   }

   if (fread (pt, 1, info.st_size, fp) != (int)(info.st_size))
   {
       perror ("Read error");
       exit (1);
   }  
   pt[info.st_size] = '\0';
   return (pt);
} 

/*
 * procData FUNCTION - Work procedure for sending files.
 * Parse the data for strings of words, and passes then to the text
 * widget. Anytime a "*word" is found, use the word as a prompt in a
 * dialog box to get the text to insert.
 */
Boolean procData ( XtPointer data )
{   
   static char *start, *ptr, *begin;
   static Boolean first = True;
   static int ToDo = TEXT;
   XmTextPosition pos = 0;   
   int i = 0;

   if (first == True)
   {
      start = (char *) data;
      begin = (char *) data;            /* keep this to free later */ 
      first = False;
   }

   switch (ToDo)
   {
      case TEXT:                        /* send file text and check for *words */
         while (start[i] != '\0')
         {
            if (start[i] != '*')
            {
               i++;                     /* any character but '*' */
               continue;
            }
            else
            {
               start[i] = '\0';         /* found a *word, * is now '\0' */
               pos = XmTextGetLastPosition (text1);
               XmTextInsert (text1, pos, start);
               i++;                     /* all text so far is now in text1 */
               start = start + i;
               ToDo = INSERT;           /* make us go to the other case next */              
               return (False);          /* reschedule the work proc */
            }
         }
         pos = XmTextGetLastPosition (text1);
         XmTextInsert (text1, pos, start);
         first = True;
         XtFree (begin);                /* Free space used by file */
         return (True);                 /* All done so don't resked work proc */

      case INSERT:                      /* Inserts text for *word */
         ptr = start;
         while ( *start != ' ' && *start != '\n')
            start++;
         *start = '\0';                 /* "*word " now is "word\0" */
         inputDiag (ptr);               /* popup the prompt dialog */
         start++;
         ToDo = TEXT;
         return (True);                 /* Don't reschedule the work proc, */
                                        /* The diag will reschedule it */

      default:
         printf ("We never get here :-( \n");
   }
   return (False);                      /* We never get here either */
}  


/*
 * tb1CB FUNCTION
 * Value Changed Callback for ToggleButton 1 (XMIT)
 */
void tb1CB (Widget w, XtPointer client_data, XtPointer call_data)
{
   XmToggleButtonCallbackStruct *ptr =
      (XmToggleButtonCallbackStruct *) call_data;

   if (ptr->set)
      send_msg (STOP_MSG, XMIT);        /* Send a XMIT msg if set */
   else
      send_msg (STOP_MSG, STANDBY);     /* Send a STANDBY msg if not set */
}


/*
 * tb2CB FUNCTION
 * Value Changed Callback for ToggleButton 2 (SIDETONE)
 */
void tb2CB (Widget w, XtPointer client_data, XtPointer call_data)

{
   XmToggleButtonCallbackStruct *ptr =
      (XmToggleButtonCallbackStruct *) call_data;

   if (ptr->set)
      send_msg (SIDETONE_MSG, ON);      /* Send a sidetone msg with "1" */
   else
      send_msg (SIDETONE_MSG, OFF);     /* Send a sidetone msg with "0" */
}


/*
 * textf2CB Function
 * Activate and Modify/Verify Callback for textf2 (WPM)
 */
void textf2CB (Widget w, XtPointer client_data, XtPointer call_data)
{
   int  i;
   String val;
   Cursor watch;
   XmAnyCallbackStruct *ptr1 = (XmAnyCallbackStruct *) call_data;
   XmTextVerifyCallbackStruct *ptr2 = (XmTextVerifyCallbackStruct *) call_data;

   switch (ptr1->reason)
   {
      case XmCR_ACTIVATE:
         XtVaGetValues (w, XmNvalue, &val, NULL);
         send_msg (WPM_MSG, val);
         watch = XCreateFontCursor (XtDisplay(shell), XC_watch);
         XDefineCursor (XtDisplay (shell), XtWindow (shell), watch);
         XtFree (val);
         break;

      case XmCR_MODIFYING_TEXT_VALUE:
         if (ptr2->text->ptr == NULL)
            break;                      /* Nothing Added */

         for (i = 0; i < ptr2->text->length; i++)
         {
            if (isdigit (ptr2->text->ptr[i]) == False || ptr2->currInsert > 1)
               ptr2->doit = False;
         }
   }
}


/*
 * scale1CB for tone
 */
void scale1CB (Widget w, XtPointer client_data, XtPointer call_data)
{
    int tone;
    char val[6];
    XmScaleCallbackStruct *pt = (XmScaleCallbackStruct *) call_data;
    
    tone = pt->value - pt->value % 10;   
    XtVaSetValues (w,
      XmNvalue, tone,
         NULL);
    sprintf (val, "%d", tone);
    send_msg (TONE_MSG, val);              
}


/*
 * input_event Function
 * Catches Button 3 presses and pops up the popup menu. Also
 * catches enter window events on textf2 and text1 and forces
 * focus to that window. 
 */
void input_event (Widget w, XtPointer client_data, XEvent *event, Boolean *cont)
{
   Widget popup = (Widget) client_data;

   if (event->type == ButtonPress && event->xbutton.button == Button3)
   {
      XmMenuPosition(popup, &(event->xbutton));
      XtManageChild(popup);
   }
   if (event->type == EnterNotify)
   {
      XmProcessTraversal(w, XmTRAVERSE_CURRENT);
   }
}


/*
 * popupCB Function
 * Popup menu callback
 */
void popupCB (Widget w, XtPointer client_data, XtPointer cbs)
{
   int reason = (int) client_data;
   int rtn;
   struct tag1 message;

   switch (reason)
   {
      case CLR_TEXT:
         XtRemoveCallback (text1, XmNmodifyVerifyCallback, text1CB, NULL);
         XtVaSetValues(text1,
            XmNvalue, "",
            XmNcursorPosition, 0,
            NULL);
         old_end = -1;
         XtAddCallback (text1, XmNmodifyVerifyCallback, text1CB, NULL);
         break;

      case CLR_SEND:
         XmTextFieldSetString(textf1, "");
         break;

      case RESET:       /* Flush the Msg Queue and clear both text fields */
         while (1)
         {              /* read and ignore all the messages in the queue */
            rtn = msgrcv (qid, (struct msgbuf *) &message,
                          MAX_WORD_LENGTH, 0, IPC_NOWAIT);
            if (rtn == -1)
            {
               if (errno == ENOMSG)
               {
                  break;  /* the queue is empty now */
               }
               else
               {
                  perror ("FLUSHING BUFFER ERROR");
               }
            }
         }

         XtRemoveCallback (text1, XmNmodifyVerifyCallback, text1CB, NULL);
         XtVaSetValues(text1,
            XmNvalue, "",
            XmNcursorPosition, 0,
            NULL);
         old_end = -1;
         XtAddCallback (text1, XmNmodifyVerifyCallback, text1CB, NULL);

         XmTextFieldSetString(textf1, "");         
         break;

      case QRT:
         quitCB (w, client_data, cbs);
         break;
         
      default:
         printf ("Do nothing\n");
   }
}


/*
 * popupHelp Function 
 */

void helpCB (Widget w, XtPointer client_data, XtPointer call_data)
{
int i;
Arg args[15];
static Widget helpDiag, helpTxt;
XmString xs;
char *textpt;

   if (helpDiag == NULL)
   {
      textpt = getFile (helppath);              /* Read the help file */

      helpDiag = XmCreateInformationDialog(shell, "helpDiag", NULL, 0);
      XtVaSetValues (XtParent(helpDiag),
         XmNtitle, "TWCW HELP FILE",
         NULL);
      XtUnmanageChild (XmMessageBoxGetChild (helpDiag, XmDIALOG_CANCEL_BUTTON));
      XtUnmanageChild (XmMessageBoxGetChild (helpDiag, XmDIALOG_HELP_BUTTON));

      xs = XmStringCreateLocalized ("TWCW HELP FILE");
      XtVaSetValues (helpDiag,
         XmNdialogStyle, XmDIALOG_MODELESS,
         XmNmessageString, xs,
         NULL);
      XmStringFree(xs);

      i=0;
      XtSetArg (args[i], XmNcolumns, 60); i++;
      XtSetArg (args[i], XmNmargin, 10); i++;
      XtSetArg (args[i], XmNrows, 10); i++;
      XtSetArg (args[i], XmNvalue, textpt); i++;
      XtSetArg (args[i], XmNeditMode, XmMULTI_LINE_EDIT); i++;
      XtSetArg (args[i], XmNscrollingPolicy, XmAUTOMATIC); i++;
      XtSetArg (args[i], XmNeditable, False); i++;
      XtSetArg (args[i], XmNresizeWidth, True); i++;
      XtSetArg (args[i], XmNautoShowCursorPosition, False); i++;
      XtSetArg (args[i], XmNcursorPositionVisible, False); i++;

      helpTxt = XmCreateScrolledText (helpDiag, "helpTxt", args, i);
      XtManageChild (helpTxt);
   }
   XtManageChild (helpDiag);
}

/*
 * Macro Actions
 */

/*
 * sendOver Action - gets the other call from shared memory,
 * gets your call, builds the "over" string, and sends it.
 * If there is no call - just send de mycall
 */
void sendOver (Widget w, XEvent *e, String args[], Cardinal *nargs)
{
   char *ch;

   if ((ch = getHisCall()) != (char *)0)        /* get his call */
   {                                            /* got it */
      procMacroText(ch);                        /* send his call, */
      shmdt ((void *)ch);                       /* detach shm   */
   }
   procMacroText ("de");                        /* send de */
   procMacroText (app_res.call);                /* and send my call */
}


/*
 * sendHisCall Action - send the call from twlog
 */
void sendHisCall (Widget w, XEvent *e, String args[], Cardinal *nargs)
{
   char *ch;

   if ((ch = getHisCall()) != (char *)0)        /* get his call */
   {                                            /* got it */
      procMacroText(ch);                        /* send his call, */
      shmdt ((void *)ch);                       /* detach shm   */
   }
}


/*
 * sendMacro Action - send all the action arguments
 */
void sendMacro (Widget w, XEvent *e, String args[], Cardinal *nargs)
{
   int i;

   for (i=0; i<(int)*nargs; i++)
   {
      procMacroText (args[i]);
   }
}


/*
 * getHisCall - attach the shared menory if running twlog and returns
 * a char * to his call
 * Used by sendOver and sendCall
 */
#define KEY 6146     /* must match Twlog's value */
#define SHMSIZE 40

char *getHisCall ()
{
   int shmid;
   void *pt;

   if ((shmid = shmget ((key_t) KEY, SHMSIZE, 0600)) < 0)
   {
      fprintf (stderr, "No twlog!\n");
      return ((char*) 0);
   }

   if ((pt = shmat (shmid, NULL, 0)) ==  (void *) -1)
   {
      perror ("twpsk - shmat failed");
      return ((char *) 0);
   }
   return ((char *)pt);
}


/*
 * procMacroText - sends a string to appendTXtext
 */
void procMacroText (char *ch)
{

#if 1   // well, this one works for lessTif and openMotif - butt ugly.
   char *str, *sp =" ";

   str = (char *) malloc (strlen (ch) + strlen (sp) + 1);
   strcpy (str, ch);
   strcat (str, sp);
   XmTextInsert (text1, XmTextGetLastPosition (text1), str);
   free (str);
#endif

#if 0    // within is a working procMacroText for openMotif
   XmTextInsert (text1,
      XmTextGetLastPosition(text1), ch);
   
   XmTextInsert (text1,
      XmTextGetLastPosition(text1) , " ");    /*but this fails with lessTif */
#endif
}


/*
 * QRT Function
 */
void quitCB (Widget w, XtPointer client_data, XtPointer cbs)
{
      cleanup();
}  
