/*
 * twcw:  A gui application that sends Morse Code 
 * Copyright (C) 1997 Ted Williams WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * Version: 1.1 -  MAR 2003
 */

/*
 *  build_widgets. Function
 *  Create all of the widgets for the interface
 */
#include "common.h"
#include "twcw.h"
#define  SPACING 5
#define POSITION 50
#define NUM_XMSTRINGS 15

void build_widgets (Widget shell, AppRes *resptr)

{
   Widget form1, label1;
   Widget menuBar, fileCas, editCas, helpCas;
   Widget fileMenu, editMenu, helpMenu;
   Widget *fileBtns=0;
   Widget textBtn, sentBtn, resetBtn, aboutBtn, helpBtn;
   Widget form2, rc1, label2, label3, label4, popup, sep;
   XmString xs;
   XmString clr_text_xs, clr_send_xs, reset_xs, quit_xs;
   XmString clr_text_acc_xs, clr_send_acc_xs, reset_acc_xs, quit_acc_xs;
   char file_acc_text[8];
   char file_acc_num[2];
   char file_acc_key[12];
   int i=0, cnt;
   Arg args[20];
   XtActionsRec actions[] =
      {
         {"sendOver", sendOver},
         {"sendHisCall", sendHisCall},
         {"sendMacro", sendMacro}
      };

   /*
    * form1 - main form
    */
   form1 = XtVaCreateWidget ("form1", xmFormWidgetClass, shell,
      XmNhorizontalSpacing, 2 * SPACING,
      XmNverticalSpacing, 2 * SPACING,
      NULL);
	   
   /*
    * label1 - callsign label
    */
   label1 = XtVaCreateManagedWidget ("label1", xmLabelWidgetClass, form1,
      XmNtopAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      NULL);

   /*
    * menu - Make the main menu bar and its cascades.
    */
   menuBar = XmCreateMenuBar (form1, "menuBar", NULL, 0);

   fileMenu = XmCreatePulldownMenu (menuBar, "filePullDown", NULL, 0);
   XtVaSetValues (fileMenu,
      XmNtearOffModel, XmTEAR_OFF_ENABLED,
      NULL);

   xs = XmStringCreateLocalized ("File");
   fileCas = XtVaCreateManagedWidget ("File", xmCascadeButtonWidgetClass,
      menuBar,
      XmNlabelString, xs,
      XmNmnemonic, 'F',
      XmNsubMenuId, fileMenu,
      NULL);
   XmStringFree (xs);

   /*
    * Count the buttons names for the File pulldown
    * and malloc space for them plus the QRT button.
    * A maximum of 9 buttons are allowed, not counting
    * the QRT button.
    */
   cnt=0;
   while (resptr->buttonNames[cnt] != NULL)
      cnt++;

   if (cnt > 9)                                    /* Too many files */
   {
      fprintf (stderr, "twcw: twcw.buttonNames has more than 9 names\n");
      exit (1);
   }

   fileBtns = (Widget *)XtMalloc (sizeof(Widget *) * (cnt+1)); 

   /*
    * Create the buttons for the File pulldown
    */
   for (i=0; i<cnt; i++)                               /* file namebuttons */
   {
      strcpy (file_acc_text, "Ctrl-");
      sprintf(file_acc_num, "%d", i+1);            
      strcat (file_acc_text, file_acc_num);
      xs = XmStringCreateLocalized (file_acc_text); /* for accelerator */
      
      strcpy (file_acc_key, "Ctrl<Key>");
      strcat (file_acc_key, file_acc_num);             /* for accelerator */

      fileBtns[i] = XtVaCreateManagedWidget(resptr->buttonNames[i],
         xmPushButtonWidgetClass, fileMenu,
         XmNaccelerator, file_acc_key,
         XmNacceleratorText, xs,
         XmNuserData, (XtPointer) resptr,
         NULL);
      XmStringFree (xs);
   }

   sep = XtVaCreateManagedWidget ("sep", xmSeparatorWidgetClass,
      fileMenu,
      NULL);
                                                   /* Add QRT button */   
   xs = XmStringCreateLocalized ("Ctrl-Q");

   fileBtns[i] = XtVaCreateManagedWidget("QRT",xmPushButtonWidgetClass,fileMenu,
      XmNaccelerator, "Ctrl<Key>Q",
      XmNacceleratorText, xs,
      XmNmnemonic, 'Q',
      XmNmnemonicCharSet, XmFONTLIST_DEFAULT_TAG,
      NULL);
      XmStringFree (xs);

   /*
    * Create the Edit Button and pulldown
    */
   editMenu = XmCreatePulldownMenu (menuBar, "editPullDown", NULL, 0);
   XtVaSetValues (editMenu,
      XmNtearOffModel, XmTEAR_OFF_ENABLED,
      NULL);

   xs = XmStringCreateLocalized ("Edit");
   editCas = XtVaCreateManagedWidget ("Edit", xmCascadeButtonWidgetClass,
      menuBar,
      XmNlabelString, xs,
      XmNmnemonic, 'E',
      XmNsubMenuId, editMenu,
      NULL);
      XmStringFree (xs);

   xs = XmStringCreateLocalized ("Ctrl-T");
   textBtn = XtVaCreateManagedWidget("Clear Text", xmPushButtonWidgetClass,
      editMenu,
      XmNacceleratorText, xs,
      NULL);
      XmStringFree (xs);

   xs = XmStringCreateLocalized ("Ctrl-S");
   sentBtn = XtVaCreateManagedWidget("Clear Sent", xmPushButtonWidgetClass,
      editMenu,
      XmNacceleratorText, xs,
      NULL);
      XmStringFree (xs);

   xs = XmStringCreateLocalized ("Ctrl-R");
   resetBtn = XtVaCreateManagedWidget("Reset", xmPushButtonWidgetClass,
      editMenu,
      XmNacceleratorText, xs,
      NULL);
      XmStringFree (xs);

   /*
    * Create the Help Button and pulldown
    */
   helpMenu = XmCreatePulldownMenu (menuBar, "helpPullDown", NULL, 0);

   xs = XmStringCreateLocalized ("Help");
   helpCas = XtVaCreateManagedWidget ("Help", xmCascadeButtonWidgetClass,
      menuBar,
      XmNlabelString, xs,
      XmNmnemonic, 'H',
      XmNsubMenuId, helpMenu,
      NULL);
      XmStringFree (xs);
     
   aboutBtn = XtVaCreateManagedWidget("About",
      xmPushButtonWidgetClass, helpMenu, NULL);
   
   helpBtn = XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, helpMenu,
      XmNuserData, (XtPointer) resptr,
      NULL);

   XtVaSetValues (menuBar,
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, label1,
      XmNleftAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNmenuHelpWidget, XtNameToWidget(menuBar, "button_1"),
      XmNmenuHelpWidget, helpCas,
      NULL);

   /*
    * form2 - form for widgets widgets on the bottom
    */
   form2 = XtVaCreateWidget ("form2", xmFormWidgetClass, form1,
      XmNmarginWidth, SPACING,
      XmNmarginHeight, SPACING,
      XmNhorizontalSpacing, SPACING,
      XmNverticalSpacing, SPACING,
      XmNtopAttachment, XmATTACH_NONE,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_FORM,
      NULL);

   /*
    * label2 - label for Sent characters
    */
   label2 = XtVaCreateManagedWidget ("label2", xmLabelWidgetClass, form2,
      XmNtopAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_FORM,     
      XmNbottomAttachment, XmATTACH_POSITION,
      XmNbottomPosition, POSITION,
      XmNrightAttachment, XmATTACH_NONE,
      NULL);

   /*
    * textf1 - textfield for sent characters
    */
   textf1 = XtVaCreateManagedWidget ("textf1", xmTextFieldWidgetClass, form2,
      XmNeditable, False,
      XmNcolumns, TEXTF1_LEN,
      XmNcursorPositionVisible, False,
      XmNtopAttachment, XmATTACH_FORM,
      XmNleftAttachment, XmATTACH_WIDGET,
      XmNleftWidget,label2,
      XmNbottomAttachment, XmATTACH_POSITION,
      XmNbottomPosition, POSITION,
      XmNrightAttachment, XmATTACH_FORM,
      XmNtraversalOn, False,
      NULL);

   /*
    * label3 - label for TONE
    */
   label3 = XtVaCreateManagedWidget ("label3", xmLabelWidgetClass, form2,
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, textf1,
      XmNleftAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, textf1,
      NULL);     

   /*
    * textf2 - textfield for WPM
    */
   textf2 = XtVaCreateManagedWidget ("textf2", xmTextFieldWidgetClass, form2,
      XmNcolumns, 3,
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, textf1,
      XmNleftAttachment, XmATTACH_NONE,
      XmNrightAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_FORM,
      NULL);

   /*
    * label4 - label for WPM
    */     
   label4 = XtVaCreateManagedWidget ("label4", xmLabelWidgetClass, form2,
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, textf1,
      XmNleftAttachment, XmATTACH_NONE,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, textf2,
      NULL);

   /*
    * rc1 - row column for XMIT and SIDETONE toggle buttons
    */
   rc1 = XtVaCreateWidget ("rc1", xmRowColumnWidgetClass, form2,
      XmNorientation, XmHORIZONTAL,     
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, textf1,
      XmNleftAttachment, XmATTACH_NONE,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, label4,
      NULL);

   /*
    * tb1 and tb2 - toggle buttons for XMIT and SIDETONE
    */
   tb1 = XtVaCreateManagedWidget ("tb1", xmToggleButtonWidgetClass, rc1,
      NULL);

   tb2 = XtVaCreateManagedWidget ("tb2", xmToggleButtonWidgetClass, rc1,
      NULL);

   /*
    * scale1 - scale for Tone  
    */
    scale1 = XtVaCreateManagedWidget ("scale1", xmScaleWidgetClass, form2,
      XmNorientation, XmHORIZONTAL,
      XmNshowValue, True,
      XmNtopAttachment, XmATTACH_WIDGET,
      XmNtopWidget, textf1,
      XmNleftAttachment, XmATTACH_WIDGET,
      XmNleftWidget, label3,
      XmNleftOffset, 8,
      XmNbottomAttachment, XmATTACH_FORM,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, rc1,
      NULL);

   /*
    * text1 - scrolled text for input text
    */
   i = 0;
   XtSetArg (args[i], XmNtopAttachment, XmATTACH_WIDGET); i++;
   XtSetArg (args[i], XmNtopWidget, menuBar); i++;
   XtSetArg (args[i], XmNrightAttachment, XmATTACH_FORM); i++;
   XtSetArg (args[i], XmNleftAttachment, XmATTACH_FORM); i++;
   XtSetArg (args[i], XmNbottomAttachment, XmATTACH_WIDGET); i++;
   XtSetArg (args[i], XmNbottomWidget, form2); i++;
   XtSetArg (args[i], XmNeditMode, XmMULTI_LINE_EDIT); i++;
   XtSetArg (args[i], XmNscrollingPolicy, XmAUTOMATIC); i++;
   XtSetArg (args[i], XmNscrollHorizontal, False); i++;
   XtSetArg (args[i], XmNwordWrap, True); i++;
   XtSetArg (args[i], XmNrows, 10); i++;
   XtSetArg (args[i], XmNscrollHorizontal, False); i++;

   text1 = XmCreateScrolledText (form1, "text1", args, i);

   XtManageChild (menuBar);
   XtManageChild (rc1);
   XtManageChild (form2);
   XtManageChild (text1);
   XtManageChild (form1);

   /*
    * Create Popup menu.
    */
   clr_text_xs = XmStringCreateLocalized ("Clear Text");
   clr_send_xs = XmStringCreateLocalized ("Clear Send");
   reset_xs = XmStringCreateLocalized ("Reset");
   quit_xs = XmStringCreateLocalized ("QRT");
   clr_text_acc_xs = XmStringCreateLocalized ("Ctrl-T");
   clr_send_acc_xs = XmStringCreateLocalized ("Ctrl-S");
   reset_acc_xs = XmStringCreateLocalized ("Ctrl-R");
   quit_acc_xs = XmStringCreateLocalized ("Ctrl-Q");

   popup = XmVaCreateSimplePopupMenu (form1, "menu", popupCB,
      XmVaPUSHBUTTON, clr_text_xs, 'T', "Ctrl<Key>t", clr_text_acc_xs,
      XmVaPUSHBUTTON, clr_send_xs, 'S', "Ctrl<Key>s", clr_send_acc_xs,
      XmVaPUSHBUTTON, reset_xs, 'R', "Ctrl<Key>r", reset_acc_xs,
      XmVaSEPARATOR,
      XmVaPUSHBUTTON, quit_xs, 'Q', "Ctrl<Key>q", quit_acc_xs,
      NULL);

   XmStringFree (clr_text_xs);
   XmStringFree (clr_send_xs);
   XmStringFree (reset_xs);
   XmStringFree (quit_xs);
   XmStringFree (clr_text_acc_xs);
   XmStringFree (clr_send_acc_xs);
   XmStringFree (reset_acc_xs);
   XmStringFree (quit_acc_xs);

   /*
    * Add Callbacks, Actions and Event Handlers
    */
   for(i=0; i<cnt; i++)               /* Filename callbacks */
   {
      XtAddCallback (fileBtns[i], XmNactivateCallback, fileCB,
                     (XtPointer) i);
   }

   XtAddCallback (fileBtns[i], XmNactivateCallback, quitCB, NULL);
   XtAddCallback (aboutBtn, XmNactivateCallback, aboutCB, NULL);
   XtAddCallback (helpBtn, XmNactivateCallback, helpCB, NULL);

   XtAddCallback (text1, XmNmodifyVerifyCallback, text1CB, NULL);
   XtAddCallback (text1, XmNvalueChangedCallback, text1CB, NULL);

   XtAddCallback (textf1, XmNmodifyVerifyCallback, textf1CB, NULL);
   XtAddCallback (textf1, XmNvalueChangedCallback, textf1CB, NULL);
   XtAddCallback (textf2, XmNactivateCallback, textf2CB, NULL);
   XtAddCallback (textf2, XmNmodifyVerifyCallback, textf2CB, NULL);
   XtAddCallback (textf2, XmNmotionVerifyCallback, textf2CB, NULL);

   XtAddCallback (tb1, XmNvalueChangedCallback, tb1CB, NULL);
   XtAddCallback (tb2, XmNvalueChangedCallback, tb2CB, NULL);

   XtAddCallback (scale1, XmNvalueChangedCallback, scale1CB, NULL);
   XtAddCallback (scale1, XmNdragCallback, scale1CB, NULL);

   XtAddCallback (textBtn, XmNactivateCallback, popupCB, (XtPointer) CLR_TEXT);
   XtAddCallback (sentBtn, XmNactivateCallback, popupCB, (XtPointer) CLR_SEND); 
   XtAddCallback (resetBtn, XmNactivateCallback, popupCB, (XtPointer) RESET); 

   XtAppAddActions (ac, actions, XtNumber(actions));

   XtAddEventHandler (form1,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (text1, EnterWindowMask, False, input_event, NULL);

/*
 * This is a hack to make the right mouse event be recognized on all the
 * children.  Well, all but the slider control in the scale trough.
 * This may go way with the latest lessTif
 */
#ifdef LESSTIF_VERSION
   printf ("doing hack for lesstif\n");
   XtAddEventHandler (menuBar,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (form2,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (rc1,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (tb1,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (tb2,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (scale1,ButtonPressMask,False,input_event,(XtPointer)popup);
   XtAddEventHandler (label1,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (label2,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (label3,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (label4,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (text1,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (textf1,ButtonPressMask,False,input_event,popup);
   XtAddEventHandler (textf2,ButtonPressMask,False,input_event,popup);
#endif

   XtRealizeWidget (shell);
}
